@@ -13,7 +13,7 @@
 #include <sys/stat.h>
 #include <ftw.h>
 
-unsigned long long
+static unsigned long long
 cvtnum(char *s)
 {
 	unsigned long long i;
@@ -43,22 +43,22 @@ int ffn(const char *, const struct stat *, int, struct FTW *);
 
 #define FBLOCKS(n)	((n)/blocksize)
 
-unsigned long long dirsize=0;		/* bytes */
-unsigned long long logsize=LOGSIZE*BLOCKSIZE;	/* bytes */
-unsigned long long fullblocks=0;	/* FS blocks */
-unsigned long long isize=0;		/* inodes bytes */
-unsigned long long blocksize=BLOCKSIZE;
-unsigned long long nslinks=0;		/* number of symbolic links */
-unsigned long long nfiles=0;		/* number of regular files */
-unsigned long long ndirs=0;		/* number of directories */
-unsigned long long nspecial=0;		/* number of special files */
-unsigned long long verbose=0;		/* verbose mode TRUE/FALSE */
+static unsigned long long dirsize=0;		/* bytes */
+static unsigned long long logsize=LOGSIZE*BLOCKSIZE;	/* bytes */
+static unsigned long long fullblocks=0;	/* FS blocks */
+static unsigned long long isize=0;		/* inodes bytes */
+static unsigned long long blocksize=BLOCKSIZE;
+static unsigned long long nslinks=0;		/* number of symbolic links */
+static unsigned long long nfiles=0;		/* number of regular files */
+static unsigned long long ndirs=0;		/* number of directories */
+static unsigned long long nspecial=0;		/* number of special files */
+static unsigned long long verbose=0;		/* verbose mode TRUE/FALSE */
 
-int __debug = 0;
-int ilog = 0;
-int elog = 0;
+static int __debug = 0;
+static int ilog = 0;
+static  int elog = 0;
 
-void
+static void
 usage(char *progname)
 {
 	fprintf(stderr,
@@ -214,7 +214,7 @@ ffn(const char *path, const struct stat *stb, int flags, struct FTW *f)
 		nfiles++;
 		break;
 	case S_IFLNK:			/* symbolic links */
-		if (stb->st_size >= (INODESIZE - (sizeof(xfs_dinode_t)+4)))
+		if (stb->st_size >= (INODESIZE - (sizeof(struct xfs_dinode)+4)))
 			fullblocks+=FBLOCKS(stb->st_size + blocksize-1);
 		nslinks++;
 		break;
