@@ -19,8 +19,8 @@ struct cmdline {
 	bool	iterate;
 };
 
-static int	ncmdline;
-struct cmdline	*cmdline;
+static int		ncmdline;
+static struct cmdline	*cmdline;
 
 static int
 compare(const void *a, const void *b)
@@ -34,6 +34,10 @@ add_command(
 	const cmdinfo_t	*ci)
 {
 	cmdtab = realloc((void *)cmdtab, ++ncmds * sizeof(*cmdtab));
+	if (!cmdtab) {
+		perror(_("adding libxcmd command"));
+		exit(1);
+	}
 	cmdtab[ncmds - 1] = *ci;
 	qsort(cmdtab, ncmds, sizeof(*cmdtab), compare);
 }
